@@ -24,7 +24,10 @@
 #ifdef HAVE_SYS_SHM_H
 
 #include <stdio.h>
+#include <stdlib.h>
+#if HAVE_MALLOC_H
 #include <malloc.h>
+#endif
 #include <string.h>
 #include <errno.h>
 #include <poll.h>
