@@ -414,12 +414,8 @@ static int write_hex(char *buf, char *str, int width)
 	void *p = &val;
 
         errno = 0;
-	val = strtol(str, NULL, 16);
-
-	if ((errno == ERANGE && (val == LONG_MAX || val == LONG_MIN))
-		|| (errno != 0 && val == 0)) {
+	if (safe_strtol_base(str, &val, 16) < 0)
 		return -EINVAL;
-        }
 
 	switch (width) {
 	case 1:
@@ -859,11 +855,11 @@ static int parse_tuple_set(snd_config_t *cfg,
 				goto err;
 			}
 
-			if ((type == SND_SOC_TPLG_TUPLE_TYPE_WORD
-					&& tuple_val > UINT_MAX)
-				|| (type == SND_SOC_TPLG_TUPLE_TYPE_SHORT
-					&& tuple_val > USHRT_MAX)
-				|| (type == SND_SOC_TPLG_TUPLE_TYPE_BYTE
+			if (/* (type == SND_SOC_TPLG_TUPLE_TYPE_WORD
+					&& tuple_val > UINT_MAX) || */
+				(type == SND_SOC_TPLG_TUPLE_TYPE_SHORT
+					&& tuple_val > USHRT_MAX) ||
+				(type == SND_SOC_TPLG_TUPLE_TYPE_BYTE
 					&& tuple_val > UCHAR_MAX)) {
 				SNDERR("tuple %s: invalid value", id);
 				goto err;
