@@ -59,6 +59,13 @@ static int get_value3(snd_use_case_mgr_t *uc_mgr,
 		      struct list_head *value_list2,
 		      struct list_head *value_list3);
 
+static int execute_sequence(snd_use_case_mgr_t *uc_mgr,
+			    struct use_case_verb *verb,
+			    struct list_head *seq,
+			    struct list_head *value_list1,
+			    struct list_head *value_list2,
+			    struct list_head *value_list3);
+
 static int execute_component_seq(snd_use_case_mgr_t *uc_mgr,
 				 struct component_sequence *cmpt_seq,
 				 struct list_head *value_list1,
@@ -66,6 +73,10 @@ static int execute_component_seq(snd_use_case_mgr_t *uc_mgr,
 				 struct list_head *value_list3,
 				 char *cdev);
 
+static inline struct use_case_device *
+	find_device(snd_use_case_mgr_t *uc_mgr, struct use_case_verb *verb,
+		    const char *device_name, int check_supported);
+
 static int check_identifier(const char *identifier, const char *prefix)
 {
 	int len;
@@ -155,7 +166,7 @@ static int read_tlv_file(unsigned int **res,
 {
 	int err = 0;
 	int fd;
-	struct stat st;
+	struct stat64 st;
 	size_t sz;
 	ssize_t sz_read;
 	struct snd_ctl_tlv *tlv;
@@ -165,7 +176,7 @@ static int read_tlv_file(unsigned int **res,
 		err = -errno;
 		return err;
 	}
-	if (fstat(fd, &st) == -1) {
+	if (fstat64(fd, &st) == -1) {
 		err = -errno;
 		goto __fail;
 	}
@@ -207,7 +218,7 @@ static int binary_file_parse(snd_ctl_elem_value_t *dst,
 {
 	int err = 0;
 	int fd;
-	struct stat st;
+	struct stat64 st;
 	size_t sz;
 	ssize_t sz_read;
 	char *res;
@@ -225,7 +236,7 @@ static int binary_file_parse(snd_ctl_elem_value_t *dst,
 		err = -errno;
 		return err;
 	}
-	if (stat(filepath, &st) == -1) {
+	if (stat64(filepath, &st) == -1) {
 		err = -errno;
 		goto __fail;
 	}
@@ -296,7 +307,7 @@ static const char *parse_uint(const char *p, const char *prefix, size_t len,
 		uc_error("unable to parse '%s'", prefix);
 		return NULL;
 	}
-	if (v < min || v > max) {
+	if ((unsigned int)v < min || (unsigned int)v > max) {
 		uc_error("value '%s' out of range %u-%u %(%ld)", min, max, v);
 		return NULL;
 	}
@@ -660,6 +671,54 @@ static int rewrite_device_value(snd_use_case_mgr_t *uc_mgr, const char *name, ch
 	return 0;
 }
 
+static int run_device_sequence(snd_use_case_mgr_t *uc_mgr, struct use_case_verb *verb,
+			       const char *name, bool enable)
+{
+	struct use_case_device *device;
+
+	if (verb == NULL) {
+		uc_error("error: enadev2 / disdev2 must be executed inside the verb context");
+		return -ENOENT;
+	}
+
+	device = find_device(uc_mgr, verb, name, 0);
+	if (device == NULL) {
+		uc_error("error: unable to find device '%s'\n", name);
+		return -ENOENT;
+	}
+
+	return execute_sequence(uc_mgr, verb,
+				enable ? &device->enable_list : &device->disable_list,
+				&device->value_list,
+				&verb->value_list,
+				&uc_mgr->value_list);
+}
+
+static int run_device_all_sequence(snd_use_case_mgr_t *uc_mgr, struct use_case_verb *verb)
+{
+	struct use_case_device *device;
+	struct list_head *pos;
+	int err;
+
+	if (verb == NULL) {
+		uc_error("error: disdevall must be executed inside the verb context");
+		return -ENOENT;
+	}
+
+	list_for_each(pos, &verb->device_list) {
+		device = list_entry(pos, struct use_case_device, list);
+
+		err = execute_sequence(uc_mgr, verb,
+					&device->disable_list,
+					&device->value_list,
+					&verb->value_list,
+					&uc_mgr->value_list);
+		if (err < 0)
+			return err;
+	}
+	return 0;
+}
+
 /**
  * \brief Execute the sequence
  * \param uc_mgr Use case manager
@@ -667,6 +726,7 @@ static int rewrite_device_value(snd_use_case_mgr_t *uc_mgr, const char *name, ch
  * \return zero on success, otherwise a negative error code
  */
 static int execute_sequence(snd_use_case_mgr_t *uc_mgr,
+			    struct use_case_verb *verb,
 			    struct list_head *seq,
 			    struct list_head *value_list1,
 			    struct list_head *value_list2,
@@ -680,6 +740,11 @@ static int execute_sequence(snd_use_case_mgr_t *uc_mgr,
 	bool ignore_error;
 	int err = 0;
 
+	if (uc_mgr->sequence_hops > 100) {
+		uc_error("error: too many inner sequences!");
+		return -EINVAL;
+	}
+	uc_mgr->sequence_hops++;
 	list_for_each(pos, seq) {
 		s = list_entry(pos, struct sequence_element, list);
 		switch (s->type) {
@@ -819,17 +884,31 @@ shell_retry:
 			if (err < 0)
 				goto __fail;
 			break;
+		case SEQUENCE_ELEMENT_TYPE_DEV_ENABLE_SEQ:
+		case SEQUENCE_ELEMENT_TYPE_DEV_DISABLE_SEQ:
+			err = run_device_sequence(uc_mgr, verb, s->data.device,
+							s->type == SEQUENCE_ELEMENT_TYPE_DEV_ENABLE_SEQ);
+			if (err < 0)
+				goto __fail;
+			break;
+		case SEQUENCE_ELEMENT_TYPE_DEV_DISABLE_ALL:
+			err = run_device_all_sequence(uc_mgr, verb);
+			if (err < 0)
+				goto __fail;
+			break;
 		default:
 			uc_error("unknown sequence command %i", s->type);
 			break;
 		}
 	}
 	free(cdev);
+	uc_mgr->sequence_hops--;
 	return 0;
       __fail_nomem:
 	err = -ENOMEM;
       __fail:
 	free(cdev);
+	uc_mgr->sequence_hops--;
 	return err;
 
 }
@@ -865,7 +944,7 @@ static int execute_component_seq(snd_use_case_mgr_t *uc_mgr,
 		seq = &device->disable_list;
 
 	/* excecute the sequence of the component dev */
-	err = execute_sequence(uc_mgr, seq,
+	err = execute_sequence(uc_mgr, uc_mgr->active_verb, seq,
 			       &device->value_list,
 			       &uc_mgr->active_verb->value_list,
 			       &uc_mgr->value_list);
@@ -927,7 +1006,7 @@ static int set_defaults(snd_use_case_mgr_t *uc_mgr)
 
 	if (uc_mgr->default_list_executed)
 		return 0;
-	err = execute_sequence(uc_mgr, &uc_mgr->default_list,
+	err = execute_sequence(uc_mgr, NULL, &uc_mgr->default_list,
 			       &uc_mgr->value_list, NULL, NULL);
 	if (err < 0) {
 		uc_error("Unable to execute default sequence");
@@ -1279,7 +1358,7 @@ static int set_verb(snd_use_case_mgr_t *uc_mgr,
 	} else {
 		seq = &verb->disable_list;
 	}
-	err = execute_sequence(uc_mgr, seq,
+	err = execute_sequence(uc_mgr, verb, seq,
 			       &verb->value_list,
 			       &uc_mgr->value_list,
 			       NULL);
@@ -1310,7 +1389,7 @@ static int set_modifier(snd_use_case_mgr_t *uc_mgr,
 	} else {
 		seq = &modifier->disable_list;
 	}
-	err = execute_sequence(uc_mgr, seq,
+	err = execute_sequence(uc_mgr, uc_mgr->active_verb, seq,
 			       &modifier->value_list,
 			       &uc_mgr->active_verb->value_list,
 			       &uc_mgr->value_list);
@@ -1344,7 +1423,7 @@ static int set_device(snd_use_case_mgr_t *uc_mgr,
 	} else {
 		seq = &device->disable_list;
 	}
-	err = execute_sequence(uc_mgr, seq,
+	err = execute_sequence(uc_mgr, uc_mgr->active_verb, seq,
 			       &device->value_list,
 			       &uc_mgr->active_verb->value_list,
 			       &uc_mgr->value_list);
@@ -1356,6 +1435,57 @@ static int set_device(snd_use_case_mgr_t *uc_mgr,
 	return err;
 }
 
+/**
+ * \brief Parse open arguments
+ * \param uc_mgr Use case manager
+ * \param name name of card to open
+ * \return the rest of the card name to open
+ */
+const char *parse_open_variables(snd_use_case_mgr_t *uc_mgr, const char *name)
+{
+	const char *end, *id;
+	char *args, *var;
+	snd_config_t *cfg, *n;
+	snd_config_iterator_t i, next;
+	char vname[128];
+	size_t l;
+	int err;
+
+	end = strstr(name, ">>>");
+	if (end == NULL)
+		return name;
+	l = end - name - 3;
+	args = alloca(l + 1);
+	strncpy(args, name + 3, l);
+	args[l] = '\0';
+
+	err = snd_config_load_string(&cfg, args, 0);
+	if (err < 0) {
+		uc_error("error: open arguments are not valid (%s)", args);
+		goto skip;
+	}
+
+	/* set arguments */
+	snd_config_for_each(i, next, cfg) {
+		n = snd_config_iterator_entry(i);
+		err = snd_config_get_id(n, &id);
+		if (err < 0)
+			goto skip;
+		err = snd_config_get_ascii(n, &var);
+		if (err < 0)
+			goto skip;
+		snprintf(vname, sizeof(vname), "@%s", id);
+		err = uc_mgr_set_variable(uc_mgr, vname, var);
+		free(var);
+		if (err < 0)
+			goto skip;
+	}
+
+skip:
+	snd_config_delete(cfg);
+	return end + 3;
+}
+
 /**
  * \brief Init sound card use case manager.
  * \param uc_mgr Returned use case manager pointer
@@ -1388,16 +1518,15 @@ int snd_use_case_mgr_open(snd_use_case_mgr_t **uc_mgr,
 		mgr->suppress_nodev_errors = 1;
 	}
 
+	if (card_name && card_name[0] == '<' && card_name[1] == '<' && card_name[2] == '<')
+		card_name = parse_open_variables(mgr, card_name);
+
 	err = uc_mgr_card_open(mgr);
 	if (err < 0) {
 		uc_mgr_free(mgr);
 		return err;
 	}
 
-	err = snd_config_top(&mgr->local_config);
-	if (err < 0)
-		goto _err;
-
 	mgr->card_name = strdup(card_name);
 	if (mgr->card_name == NULL) {
 		err = -ENOMEM;
@@ -1504,7 +1633,7 @@ static int dismantle_use_case(snd_use_case_mgr_t *uc_mgr)
 	}
 	uc_mgr->active_verb = NULL;
 
-	err = execute_sequence(uc_mgr, &uc_mgr->default_list,
+	err = execute_sequence(uc_mgr, NULL, &uc_mgr->default_list,
 			       &uc_mgr->value_list, NULL, NULL);
 	
 	return err;
@@ -1520,7 +1649,7 @@ int snd_use_case_mgr_reset(snd_use_case_mgr_t *uc_mgr)
 	int err;
 
 	pthread_mutex_lock(&uc_mgr->mutex);
-	err = execute_sequence(uc_mgr, &uc_mgr->default_list,
+	err = execute_sequence(uc_mgr, NULL, &uc_mgr->default_list,
 			       &uc_mgr->value_list, NULL, NULL);
 	INIT_LIST_HEAD(&uc_mgr->active_modifiers);
 	INIT_LIST_HEAD(&uc_mgr->active_devices);
@@ -2347,7 +2476,7 @@ static int set_fixedboot_user(snd_use_case_mgr_t *uc_mgr,
 	}
 	if (list_empty(&uc_mgr->fixedboot_list))
 		return -ENOENT;
-	err = execute_sequence(uc_mgr, &uc_mgr->fixedboot_list,
+	err = execute_sequence(uc_mgr, NULL, &uc_mgr->fixedboot_list,
 			       &uc_mgr->value_list, NULL, NULL);
 	if (err < 0) {
 		uc_error("Unable to execute force boot sequence");
@@ -2367,7 +2496,7 @@ static int set_boot_user(snd_use_case_mgr_t *uc_mgr,
 	}
 	if (list_empty(&uc_mgr->boot_list))
 		return -ENOENT;
-	err = execute_sequence(uc_mgr, &uc_mgr->boot_list,
+	err = execute_sequence(uc_mgr, NULL, &uc_mgr->boot_list,
 			       &uc_mgr->value_list, NULL, NULL);
 	if (err < 0) {
 		uc_error("Unable to execute boot sequence");
@@ -2396,7 +2525,8 @@ static int handle_transition_verb(snd_use_case_mgr_t *uc_mgr,
 	list_for_each(pos, &uc_mgr->active_verb->transition_list) {
 		trans = list_entry(pos, struct transition_sequence, list);
 		if (strcmp(trans->name, new_verb->name) == 0) {
-			err = execute_sequence(uc_mgr, &trans->transition_list,
+			err = execute_sequence(uc_mgr, uc_mgr->active_verb,
+					       &trans->transition_list,
 					       &uc_mgr->active_verb->value_list,
 					       &uc_mgr->value_list,
 					       NULL);
@@ -2507,7 +2637,8 @@ static int switch_device(snd_use_case_mgr_t *uc_mgr,
 	list_for_each(pos, &xold->transition_list) {
 		trans = list_entry(pos, struct transition_sequence, list);
 		if (strcmp(trans->name, new_device) == 0) {
-			err = execute_sequence(uc_mgr, &trans->transition_list,
+			err = execute_sequence(uc_mgr, uc_mgr->active_verb,
+					       &trans->transition_list,
 					       &xold->value_list,
 					       &uc_mgr->active_verb->value_list,
 					       &uc_mgr->value_list);
@@ -2559,7 +2690,8 @@ static int switch_modifier(snd_use_case_mgr_t *uc_mgr,
 	list_for_each(pos, &xold->transition_list) {
 		trans = list_entry(pos, struct transition_sequence, list);
 		if (strcmp(trans->name, new_modifier) == 0) {
-			err = execute_sequence(uc_mgr, &trans->transition_list,
+			err = execute_sequence(uc_mgr, uc_mgr->active_verb,
+					       &trans->transition_list,
 					       &xold->value_list,
 					       &uc_mgr->active_verb->value_list,
 					       &uc_mgr->value_list);
